@@ -2,6 +2,7 @@ package org.gcube.portlets.user.td.gwtservice.shared.monitor;
 
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
+import java.util.List;
 
 import javax.servlet.http.HttpSession;
 
@@ -9,18 +10,23 @@ import org.gcube.application.framework.core.session.ASLSession;
 import org.gcube.data.analysis.tabulardata.commons.utils.AuthorizationProvider;
 import org.gcube.data.analysis.tabulardata.commons.utils.AuthorizationToken;
 import org.gcube.data.analysis.tabulardata.commons.webservice.types.TaskStatus;
+import org.gcube.data.analysis.tabulardata.model.metadata.common.TableDescriptorMetadata;
 import org.gcube.data.analysis.tabulardata.model.metadata.table.DatasetViewTableMetadata;
 import org.gcube.data.analysis.tabulardata.model.metadata.table.ExportMetadata;
 import org.gcube.data.analysis.tabulardata.model.table.Table;
+import org.gcube.data.analysis.tabulardata.model.table.TableId;
 import org.gcube.data.analysis.tabulardata.service.TabularDataService;
 import org.gcube.data.analysis.tabulardata.service.impl.TabularDataServiceFactory;
 import org.gcube.data.analysis.tabulardata.service.operation.Job;
 import org.gcube.data.analysis.tabulardata.service.operation.Task;
+import org.gcube.data.analysis.tabulardata.service.operation.TaskResult;
 import org.gcube.data.analysis.tabulardata.service.operation.ValidationJob;
 import org.gcube.data.analysis.tabulardata.service.tabular.TabularResource;
 import org.gcube.data.analysis.tabulardata.service.tabular.TabularResourceId;
+import org.gcube.data.analysis.tabulardata.service.tabular.metadata.NameMetadata;
 import org.gcube.portlets.user.td.gwtservice.server.SessionUtil;
 import org.gcube.portlets.user.td.gwtservice.server.storage.FilesStorage;
+import org.gcube.portlets.user.td.gwtservice.server.trservice.JobClassifierMap;
 import org.gcube.portlets.user.td.gwtservice.server.trservice.TabularResourceTypeMap;
 import org.gcube.portlets.user.td.gwtservice.server.trservice.TaskStateMap;
 import org.gcube.portlets.user.td.gwtservice.server.trservice.WorkerStateMap;
@@ -29,11 +35,10 @@ import org.gcube.portlets.user.td.gwtservice.shared.exception.TDGWTServiceExcept
 import org.gcube.portlets.user.td.gwtservice.shared.exception.TDGWTSessionExpiredException;
 import org.gcube.portlets.user.td.gwtservice.shared.json.JSONExportSession;
 import org.gcube.portlets.user.td.gwtservice.shared.task.JobS;
-import org.gcube.portlets.user.td.gwtservice.shared.task.State;
+import org.gcube.portlets.user.td.gwtservice.shared.task.JobSClassifier;
 import org.gcube.portlets.user.td.gwtservice.shared.task.TaskS;
 import org.gcube.portlets.user.td.gwtservice.shared.task.TaskWrapper;
 import org.gcube.portlets.user.td.gwtservice.shared.task.ValidationsJobS;
-import org.gcube.portlets.user.td.gwtservice.shared.tr.table.metadata.TabExportMetadata;
 import org.gcube.portlets.user.td.widgetcommonevent.shared.TRId;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -46,16 +51,15 @@ import org.slf4j.LoggerFactory;
  */
 public class BackgroundOperationMonitorCreator {
 	private static final String SECURITY_EXCEPTION_RIGHTS = "Security exception, you don't have the required rights!";
-	
+
 	private static Logger logger = LoggerFactory
 			.getLogger(BackgroundOperationMonitorCreator.class);
 
-	private static SimpleDateFormat sdf = new SimpleDateFormat(
-			"yyyy-MM-dd HH:mm");
+	//private static SimpleDateFormat sdf = new SimpleDateFormat(
+	//		"yyyy-MM-dd HH:mm");
 
 	@SuppressWarnings("unused")
-	private static SimpleDateFormat sdfDate = new SimpleDateFormat(
-			"yyyy-MM-dd");
+	private static SimpleDateFormat sdfDate = new SimpleDateFormat("yyyy-MM-dd");
 
 	protected HttpSession session;
 	protected TaskWrapper taskWrapper;
@@ -75,7 +79,8 @@ public class BackgroundOperationMonitorCreator {
 		this.aslSession = aslSession;
 		this.taskWrapper = taskWrapper;
 		this.operationMonitorSession = operationMonitorSession;
-
+		logger.debug("BackgroundOperationMonitorCreator: " + taskWrapper + " "
+				+ operationMonitorSession);
 	}
 
 	/**
@@ -103,11 +108,10 @@ public class BackgroundOperationMonitorCreator {
 				Task task = taskWrapper.getTask();
 				task.abort();
 				TaskS taskS = createTaskS();
-				taskS.setState(State.ABORTED);
+				// taskS.setState(State.ABORTED);
 				operationMonitor.setTask(taskS);
-				operationMonitor.setAbort(true);
-				SessionUtil.removeTaskInBackground(session, taskWrapper);
-				SessionUtil.setAbortedTasks(session, taskWrapper);
+				// operationMonitor.setAbort(true);
+				SessionUtil.setTaskInBackground(session, taskWrapper);
 				postOperation(operationMonitor);
 			} else {
 				if (operationMonitorSession.isHidden()) {
@@ -147,7 +151,7 @@ public class BackgroundOperationMonitorCreator {
 		return operationMonitor;
 	}
 
-	protected TaskS createTaskS() {
+	protected TaskS createTaskS() throws TDGWTServiceException {
 		TaskS taskS = new TaskS();
 
 		ArrayList<JobS> jobSList = new ArrayList<JobS>();
@@ -165,9 +169,11 @@ public class BackgroundOperationMonitorCreator {
 				validationsJobS.add(validationJ);
 				j++;
 			}
-
+			
+			JobSClassifier jobClassifier=JobClassifierMap.map(job.getJobClassifier());
+			
 			JobS jobS = new JobS(String.valueOf(i), job.getProgress(),
-					job.getHumaReadableStatus(), job.getDescription(),
+					job.getHumaReadableStatus(), jobClassifier, job.getDescription(),
 					WorkerStateMap.map(job.getStatus()), job.getErrorMessage(),
 					validationsJobS);
 
@@ -176,20 +182,49 @@ public class BackgroundOperationMonitorCreator {
 
 		}
 
-		taskS = new TaskS(taskWrapper.getTask().getId().getValue(), taskWrapper
-				.getTask().getProgress(), TaskStateMap.map(taskWrapper
-				.getTask().getStatus()), taskWrapper.getTask().getErrorCause(),
-				taskWrapper.getTask().getSubmitter(), taskWrapper.getTask()
-						.getStartTime(), taskWrapper.getTask().getEndTime(),
-				jobSList);
+		try {
 
+			ArrayList<TRId> collateralTRIds = new ArrayList<TRId>();
+			TaskResult taskResult = taskWrapper.getTask().getResult();
+			if (taskResult != null) {
+				List<TableId> collaterals = taskResult.getCollateralTables();
+				for (TableId tId : collaterals) {
+					String tabulRId = retrieveTabularResourceIdFromTable(tId);
+					TRId tabularRId = new TRId(tabulRId);
+					tabularRId.setTableId(String.valueOf(tId.getValue()));
+					collateralTRIds.add(tabularRId);
+				}
+
+			}
+
+			taskS = new TaskS(taskWrapper.getTask().getId().getValue(),
+					taskWrapper.getTask().getProgress(),
+					TaskStateMap.map(taskWrapper.getTask().getStatus()),
+					taskWrapper.getTask().getErrorCause(), taskWrapper
+							.getTask().getSubmitter(), taskWrapper.getTask()
+							.getStartTime(),
+					taskWrapper.getTask().getEndTime(), jobSList,
+					collateralTRIds, String.valueOf(taskWrapper.getTask()
+							.getTabularResourceId().getValue()));
+
+		} catch (Throwable e) {
+			logger.error("error retrieving information about the task, "
+					+ e.getLocalizedMessage());
+			e.printStackTrace();
+			throw new TDGWTServiceException(
+					"error retrieving information about the task, "
+							+ e.getLocalizedMessage());
+		}
 		logger.debug("Retrieved task information");
+
 		return taskS;
 	}
 
 	protected void postOperation(OperationMonitor operationMonitor)
 			throws TDGWTServiceException {
 
+		retrieveNameOfTabularResourceOnWhicWasLaunchedTheTask(operationMonitor);
+
 		switch (operationMonitor.getTask().getState()) {
 		case FAILED:
 			Throwable errorCause = taskWrapper.getTask().getErrorCause();
@@ -213,6 +248,9 @@ public class BackgroundOperationMonitorCreator {
 		case GENERATING_VIEW:
 			break;
 		case ABORTED:
+			operationMonitor.setAbort(true);
+			SessionUtil.removeTaskInBackground(session, taskWrapper);
+			SessionUtil.setAbortedTasks(session, taskWrapper);
 			break;
 		case STOPPED:
 			logger.debug("Task Result:" + taskWrapper.getTask().getResult());
@@ -223,12 +261,13 @@ public class BackgroundOperationMonitorCreator {
 		default:
 			break;
 		}
+
 	}
 
 	protected void updateInformations(OperationMonitor operationMonitor)
 			throws TDGWTServiceException {
 		TRId trId;
-		//TabResource tabResource;
+		// TabResource tabResource;
 		Table table;
 		ExportMetadata exportMetadata;
 
@@ -241,21 +280,21 @@ public class BackgroundOperationMonitorCreator {
 
 			operationMonitor.setTrId(SessionUtil.getTRId(session));
 
-			TabExportMetadata trExportMetadata;
+			/*TabExportMetadata trExportMetadata;
 			trExportMetadata = new TabExportMetadata();
 			trExportMetadata.setUrl(exportMetadata.getUri());
 			trExportMetadata.setDestinationType(exportMetadata
 					.getDestinationType());
 			trExportMetadata.setExportDate(sdf.format(exportMetadata
-					.getExportDate()));
+					.getExportDate()));*/
 
 			saveCSVExportInDestination(exportMetadata);
 			break;
 		case SDMXExport:
 			table = taskWrapper.getTask().getResult().getPrimaryTable();
 			logger.debug("Table retrived: " + table.toString());
-			exportMetadata = table.getMetadata(ExportMetadata.class);
-			logger.debug("ExportMetadata: " + exportMetadata);
+			//exportMetadata = table.getMetadata(ExportMetadata.class);
+			//logger.debug("ExportMetadata: " + exportMetadata);
 			operationMonitor.setTrId(SessionUtil.getTRId(session));
 			break;
 		case JSONExport:
@@ -265,33 +304,23 @@ public class BackgroundOperationMonitorCreator {
 			logger.debug("ExportMetadata: " + exportMetadata);
 
 			operationMonitor.setTrId(SessionUtil.getTRId(session));
-			
-			trExportMetadata = new TabExportMetadata();
+
+			/*trExportMetadata = new TabExportMetadata();
 			trExportMetadata.setUrl(exportMetadata.getUri());
 			trExportMetadata.setDestinationType(exportMetadata
 					.getDestinationType());
 			trExportMetadata.setExportDate(sdf.format(exportMetadata
-					.getExportDate()));
+					.getExportDate()));*/
 
 			saveJSONExportInDestination(exportMetadata);
-			break;		
+			break;
 		default:
 			trId = new TRId();
 			trId.setId(taskWrapper.getTrId().getId());
 			trId = retrieveTabularResourceBasicData(trId);
 
 			operationMonitor.setTrId(trId);
-			/*tabResource = SessionUtil.getTabResource(session);
-			logger.debug("CurrentTabResource :" + tabResource);
-			if (tabResource != null) {
-				tabResource.setTrId(trId);
-				SessionUtil.setTabResource(session, tabResource);
-			} else {
-				tabResource = new TabResource();
-				tabResource.setTrId(trId);
-				SessionUtil.setTabResource(session, tabResource);
-			}
-			SessionUtil.setTRId(session, trId);*/
+
 			break;
 
 		}
@@ -327,14 +356,18 @@ public class BackgroundOperationMonitorCreator {
 
 			TRId newTRId;
 			if (viewTable == null) {
-				newTRId = new TRId(String.valueOf(tr.getId().getValue()),
-						TabularResourceTypeMap.map(tr.getTabularResourceType()), tr.getTableType(), String.valueOf(table.getId()
-										.getValue()), table.getTableType().getName());
+				newTRId = new TRId(
+						String.valueOf(tr.getId().getValue()),
+						TabularResourceTypeMap.map(tr.getTabularResourceType()),
+						tr.getTableType(), String.valueOf(table.getId()
+								.getValue()), table.getTableType().getName());
 
 			} else {
-				newTRId = new TRId(String.valueOf(tr.getId().getValue()),
-						TabularResourceTypeMap.map(tr.getTabularResourceType()), tr.getTableType(), String.valueOf(viewTable.getId()
-										.getValue()), viewTable.getTableType()
+				newTRId = new TRId(
+						String.valueOf(tr.getId().getValue()),
+						TabularResourceTypeMap.map(tr.getTabularResourceType()),
+						tr.getTableType(), String.valueOf(viewTable.getId()
+								.getValue()), viewTable.getTableType()
 								.getName(), String.valueOf(table.getId()
 								.getValue()), true);
 
@@ -347,11 +380,10 @@ public class BackgroundOperationMonitorCreator {
 			throw e;
 		} catch (SecurityException e) {
 			e.printStackTrace();
-			throw new TDGWTServiceException(
-					SECURITY_EXCEPTION_RIGHTS);
+			throw new TDGWTServiceException(SECURITY_EXCEPTION_RIGHTS);
 		} catch (Throwable e) {
 			e.printStackTrace();
-			throw new TDGWTServiceException("Error in Client Library Request: "
+			throw new TDGWTServiceException("Error on Service: "
 					+ e.getLocalizedMessage());
 		}
 	}
@@ -379,16 +411,18 @@ public class BackgroundOperationMonitorCreator {
 			boolean end = SessionUtil.getCSVExportEnd(session);
 			if (end == false) {
 				SessionUtil.setCSVExportEnd(session, true);
-				FilesStorage storage = new FilesStorage();
-				logger.debug("Create Item On Workspace: [ uri: "
+				logger.debug("Create Item On Workspace: [ storageId: "
 						+ exportMetadata.getUri() + " ,user: " + user
 						+ " ,fileName: " + exportSession.getFileName()
 						+ " ,fileDescription: "
 						+ exportSession.getFileDescription()
 						+ " ,mimetype: text/csv" + " ,folder: "
 						+ exportSession.getItemId() + "]");
-				storage.createItemOnWorkspace(exportMetadata.getUri(), user,
-						exportSession.getFileName(),
+				
+				FilesStorage storage = new FilesStorage();
+				
+				storage.createItemOnWorkspaceByStorageId(exportMetadata.getUri(), user,
+						exportSession.getFileName()+".csv",
 						exportSession.getFileDescription(), "text/csv",
 						exportSession.getItemId());
 
@@ -425,16 +459,19 @@ public class BackgroundOperationMonitorCreator {
 			boolean end = SessionUtil.getJSONExportEnd(session);
 			if (end == false) {
 				SessionUtil.setJSONExportEnd(session, true);
-				FilesStorage storage = new FilesStorage();
-				logger.debug("Create Item On Workspace: [ uri: "
+				
+				logger.debug("Create Item On Workspace: [ storageId: "
 						+ exportMetadata.getUri() + " ,user: " + user
 						+ " ,fileName: " + exportSession.getFileName()
 						+ " ,fileDescription: "
 						+ exportSession.getFileDescription()
 						+ " ,mimetype: application/json" + " ,folder: "
 						+ exportSession.getItemId() + "]");
-				storage.createItemOnWorkspace(exportMetadata.getUri(), user,
-						exportSession.getFileName(),
+				
+				FilesStorage storage = new FilesStorage();
+				
+				storage.createItemOnWorkspaceByStorageId(exportMetadata.getUri(), user,
+						exportSession.getFileName()+".json",
 						exportSession.getFileDescription(), "application/json",
 						exportSession.getItemId());
 
@@ -447,5 +484,86 @@ public class BackgroundOperationMonitorCreator {
 					"Error in export json: no destination present");
 		}
 	}
-	
+
+	private String retrieveTabularResourceIdFromTable(TableId tableId)
+			throws TDGWTServiceException {
+		try {
+			AuthorizationProvider.instance.set(new AuthorizationToken(
+					aslSession.getUsername(), aslSession.getScope()));
+			TabularDataService service = TabularDataServiceFactory.getService();
+
+			Table table = service.getTable(tableId);
+
+			if (table.contains(TableDescriptorMetadata.class)) {
+				TableDescriptorMetadata tdm = table
+						.getMetadata(TableDescriptorMetadata.class);
+				return String.valueOf(tdm.getRefId());
+			} else {
+				throw new TDGWTServiceException(
+						"No TableDescriptorMetadata present in tableId: "
+								+ tableId);
+			}
+
+		} catch (Exception e) {
+			throw new TDGWTServiceException(e.getLocalizedMessage());
+		}
+
+	}
+
+	private void retrieveNameOfTabularResourceOnWhicWasLaunchedTheTask(
+			OperationMonitor operationMonitor) throws TDGWTServiceException {
+		try {
+			AuthorizationProvider.instance.set(new AuthorizationToken(
+					aslSession.getUsername(), aslSession.getScope()));
+			TabularDataService service = TabularDataServiceFactory.getService();
+
+			if (operationMonitor == null || operationMonitor.getTask() == null) {
+				logger.debug("No valid operation monitor: " + operationMonitor);
+				operationMonitor.setTabularResourceName("NoName");
+				return;
+			}
+
+			String tabRId = operationMonitor.getTask().getTabularResourceId();
+			logger.debug("TabularResourceId: " + tabRId);
+			if (tabRId == null || tabRId.isEmpty()) {
+				logger.debug("No valid tabular resource id: " + tabRId);
+				operationMonitor.setTabularResourceName("NoName");
+				return;
+			}
+
+			Long tabIdL = null;
+			try {
+				tabIdL = new Long(tabRId);
+			} catch (NumberFormatException e) {
+				logger.debug("No valid tabular resource id: " + tabRId + " "
+						+ e.getLocalizedMessage());
+				operationMonitor.setTabularResourceName("NoName");
+				return;
+			}
+
+			if (tabIdL == 0) {
+				logger.debug("No valid tabular resource id: " + tabIdL);
+				operationMonitor.setTabularResourceName("NoName");
+				return;
+			}
+			
+			logger.debug("retrieveNameOfTabularResourceOnWhicWasLaunchedTheTask(): on TabularResourceId "
+					+ tabIdL);
+
+			TabularResourceId tabularResourceId = new TabularResourceId(tabIdL);
+			TabularResource tr = service.getTabularResource(tabularResourceId);
+
+			if (tr.contains(NameMetadata.class)) {
+				NameMetadata nm = tr.getMetadata(NameMetadata.class);
+				operationMonitor.setTabularResourceName(nm.getValue());
+			} else {
+				operationMonitor.setTabularResourceName("NoName");
+			}
+
+		} catch (Exception e) {
+			throw new TDGWTServiceException(e.getLocalizedMessage());
+		}
+
+	}
+
 }
